#include "mem_allocator.h"
#include "../../../common/aliases/aliases.h"
#include <stdlib.h>

void *wapp_mem_allocator_alloc(const Allocator *allocator, u64 size) {
  if (!allocator || !(allocator->alloc)) {
    return NULL;
  }

  return allocator->alloc(size, allocator->obj);
}

void *wapp_mem_allocator_alloc_aligned(const Allocator *allocator, u64 size, u64 alignment) {
  if (!allocator || !(allocator->alloc_aligned)) {
    return NULL;
  }

  return allocator->alloc_aligned(size, alignment, allocator->obj);
}

void *wapp_mem_allocator_realloc(const Allocator *allocator, void *ptr, u64 old_size, u64 new_size) {
  if (!allocator || !(allocator->realloc)) {
    return NULL;
  }

  return allocator->realloc(ptr, old_size, new_size, allocator->obj);
}

void *wapp_mem_allocator_realloc_aligned(const Allocator *allocator, void *ptr, u64 old_size,
                                         u64 new_size, u64 alignment) {
  if (!allocator || !(allocator->realloc_aligned)) {
    return NULL;
  }

  return allocator->realloc_aligned(ptr, old_size, new_size, alignment, allocator->obj);
}

void wapp_mem_allocator_free(const Allocator *allocator, void **ptr, u64 size) {
  if (!allocator || !(allocator->free)) {
    return;
  }

  allocator->free(ptr, size, allocator->obj);
}
